#!/bin/bash
#

set -o nounset
set -o errexit
set -x

# =============================================================================
# Author - F. Massonnet
# Purpose- Prepare DFS5.2 forcing set to create perturbations later
# What the script does:
#   1) Make daily means in the case atmospheric data is at higher frequency
#   2) Remove leap days if any
#   3) Computes year-to-year differences
#
# The year-to-year differences are then used to generate perturbations,
# in a second script.
#
# History - February 2016: creation for T2 perturbation
#         - March    2016: sorting problems with values beyond bounds
#         - April    2016: extension to other variables

# =============================================================================

# Which variable of the DFS5.2 forcing set has to be perturbed?
# Possibilities: t2, q2, u10, v10, qlw, qsw, snow or precip
var=t2

# First and last years for which perturbations have to be created.
# yearb = 1958 and yeare = 2015 are recommended choices since the forcing
# spans these years.

yearb=2014
yeare=2015

# Directory of the source of the data (i.e., the original files)
sourcedir=/esnas/releases/fg/ocean/DFS5.2/

# Working directory
workdir=/esnas/scratch/$USER/TMP/TMP_${var}_54

# =============================================================================
# Script Starts
# =============================================================================

mkdir -p $workdir
cd    $workdir
echo "Workdir is $workdir"

case ${var} in 
  t2)
    min=100.0 	# Min and max values allowed
    max=400.0
    freq=3hour  # Frequency of availability
    fvar=${var} # Name of the variable in the NetCDF
    ;;
  q2)
    min=0.0
    max=0.1
    freq=3hour
    fvar=${var}
    ;;
  u10)
    min=-100.0
    max=100.0
    freq=3hour
    fvar=${var}
    ;;
  v10)
    min=-100.0
    max=100.0
    freq=3hour
    fvar=${var}
    ;;
  qsw)
    min=0.0
    max=1000.0
    freq=1day
    fvar=radsw
    ;;
  qlw)
    min=0.0
    max=1000.0
    freq=1day
    fvar=radlw
    ;;
  snow)
    min=0.0
    max=0.001
    freq=1day
    fvar=${var}
    ;;
  precip)
    min=0.0
    max=0.01
    freq=1day 
    fvar=${var}
    ;;
  *)
  echo "Variable $var unknown"
  exit
esac

for year in `seq ${yearb} ${yeare}`
do
  echo "Year $year / $yeare"

  # Reset valid range. Otherwise, the CDO command settaxis messes up and several points are reported as 
  # missing values. This then makes NEMO crash.
  ncatted -O -a valid_range,${fvar},m,f,"${min}, ${max}" ${sourcedir}/${var}_DFS5.2_${year}.nc tmp.${year}.nc 

  cdo settaxis,${year}-01-01,00:00,${freq} tmp.${year}.nc ${var}_DFS5.2_${year}.nc
  if [ ${freq} = 3hour ]
  then
    cdo daymean ${var}_DFS5.2_${year}.nc ${var}_DFS5.2_day_${year}.nc
  else
    cp ${var}_DFS5.2_${year}.nc ${var}_DFS5.2_day_${year}.nc
  fi

  # Cope with leap years
  ndays=`cdo ntime ${var}_DFS5.2_day_${year}.nc`

  if [[ $ndays = 366 ]]
  then
    cdo delete,month=2,day=29 ${var}_DFS5.2_day_${year}.nc ${var}_DFS5.2_day_${year}.nc.X
    mv ${var}_DFS5.2_day_${year}.nc.X ${var}_DFS5.2_day_${year}.nc
  else
    if [[ $ndays != 365 ]]
    then
      echo "Problem: nb of days is neither 365 nor 366"
      exit
    fi  
  fi

  if [[ $year -ge $(( ${yearb} + 1 )) ]]
  then
    cdo sub ${var}_DFS5.2_day_${year}.nc ${var}_DFS5.2_day_$(( ${year} - 1 )).nc diff_${var}_DFS5.2_day_${year}-$(( ${year} - 1 )).nc
  fi

  rm -f tmp.${year}.nc
done
